Module: MerbPaginate::Finders::Activerecord::ClassMethods
- Includes:
- GenericOrmMethods
- Defined in:
- lib/merb_paginate/finders/activerecord.rb
Overview
Paginating finders for ActiveRecord models
WillPaginate adds paginate and per_page methods to ActiveRecord::Base
class methods and associations. It also hooks into method_missing to
intercept pagination calls to dynamic finders such as
paginate_by_user_id and translate them to ordinary finders
(+find_all_by_user_id+ in this case).
In short, paginating finders are equivalent to ActiveRecord finders; the only difference is that we start with "paginate" instead of "find" and that :page is required parameter:
@posts = Post.paginate :all, :page => params[:page], :order => 'created_at DESC'
In paginating finders, "all" is implicit. There is no sense in paginating a single record, right? So, you can drop the :all argument:
Post.paginate(...) => Post.find :all
Post.paginate_all_by_something => Post.find_all_by_something
Post.paginate_by_something => Post.find_all_by_something
The importance of the :order parameter
In ActiveRecord finders, :order parameter specifies columns for the ORDER BY clause in SQL. It is important to have it, since pagination only makes sense with ordered sets. Without the ORDER BY clause, databases aren't required to do consistent ordering when performing SELECT queries; this is especially true for PostgreSQL.
Instance Method Summary collapse
-
#paginate(*args, &block) ⇒ Object
This is the main paginating finder.
-
#paginate_by_sql(sql, options) ⇒ Object
Wraps
find_by_sqlby simply adding LIMIT and OFFSET to your SQL string based on the params otherwise used by paginating finds:pageandper_page.
Instance Method Details
#paginate(*args, &block) ⇒ Object
This is the main paginating finder.
Special parameters for paginating finders
- :page -- REQUIRED, but defaults to 1 if false or nil
- :per_page -- defaults to CurrentModel.per_page (which is 30 if not overridden)
- :total_entries -- use only if you manually count total entries
- :count -- additional options that are passed on to
count - :finder -- name of the ActiveRecord finder used (default: "find")
All other options (+conditions+, order, ...) are forwarded to find
and count calls.
59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 |
# File 'lib/merb_paginate/finders/activerecord.rb', line 59 def paginate(*args, &block) = args.pop page, per_page, total_entries = () finder = ([:finder] || 'find').to_s if finder == 'find' # an array of IDs may have been given: total_entries ||= (Array === args.first and args.first.size) # :all is implicit args.unshift(:all) if args.empty? end WillPaginate::Collection.create(page, per_page, total_entries) do |pager| = .except :page, :per_page, :total_entries, :finder = .except(:count).update(:offset => pager.offset, :limit => pager.per_page) args << # @options_from_last_find = nil pager.replace send(finder, *args, &block) # magic counting for user convenience: pager.total_entries = wp_count(, args, finder) unless pager.total_entries end end |
#paginate_by_sql(sql, options) ⇒ Object
Wraps find_by_sql by simply adding LIMIT and OFFSET to your SQL string
based on the params otherwise used by paginating finds: page and
per_page.
Example:
@developers = Developer.paginate_by_sql ['select * from developers where salary > ?', 80000],
:page => params[:page], :per_page => 3
A query for counting rows will automatically be generated if you don't supply :total_entries. If you experience problems with this generated SQL, you might want to perform the count manually in your application.
98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 |
# File 'lib/merb_paginate/finders/activerecord.rb', line 98 def paginate_by_sql(sql, ) WillPaginate::Collection.create(*()) do |pager| query = sanitize_sql(sql) original_query = query.dup # add limit, offset add_limit! query, :offset => pager.offset, :limit => pager.per_page # perfom the find pager.replace find_by_sql(query) unless pager.total_entries count_query = original_query.sub /\bORDER\s+BY\s+[\w`,\s]+$/mi, '' count_query = "SELECT COUNT(*) FROM (#{count_query}) AS count_table" # perform the count query pager.total_entries = count_by_sql(count_query) end end end |